import pygame, math,random
from utility import*
from shotbase import StandardBullet
from vector import Vector
import particlebase, shipbase

class Platform(shipbase.FlyingObject):
    def __init__(self,x,y, team):
        super(Platform, self).__init__(x, y, 500, 500, team)

        self.range = 1000

        self.health = 1000

        self.cr = 200

        self.fire_left = False
        self.fire_right = False
        self.fire_up = False
        self.fire_down = False

        self.dl = 0.0
        self.dr = 0.0
        self.du = 0.0
        self.dd = 0.0

        self.gundelay = 0.05

        self.falling = False
        self.fall = 1.0

        self.frame = 0.0
        self.framerate = 20.0
        
    def update(self, others, game, delta):

        self.frame += self.framerate*delta
        if self.frame >= 4:
            self.frame = 0
        
        if self.falling:
            self.fall -= 0.2*delta

        if self.fall <= 0.02:
            self.remove = True
            #game.SpawnPart(particlebase.Flash(0.3))
            game.snd.play_sound("explosion")
            game.SpawnPart(particlebase.Explosion(self.x,self.y,50,2.0,-80))

        if not self.falling:
            self.fire_left = False
            self.fire_right = False
            self.fire_up = False
            self.fire_down = False
            
            for i in others:
                if i.team != self.team:
                    if Distance(self.x,self.y, i.x,i.y) < self.range:
                        s = Vector([i.x-self.x,i.y-self.y]).safe_normalised()
                        #check for left (1,0)
                        lu = Vector([1,1])
                        ru = Vector([-1,1])
                        ld = Vector([1,-1])
                        rd = Vector([-1,-1])
                        
                        if lu.dot(s) > 0 and ld.dot(s) > 0:
                            self.fire_left = True

                        if ru.dot(s) > 0 and rd.dot(s) > 0:
                            self.fire_right = True

                        if lu.dot(s) > 0 and ru.dot(s) > 0:
                            self.fire_up = True

                        if ld.dot(s) > 0 and rd.dot(s) > 0:
                            self.fire_down = True

            self.dl -= delta
            self.dr -= delta
            self.du -= delta
            self.dd -= delta

            if self.fire_left == True:
                if self.dl <= 0:
                    self.dl = self.gundelay
                    x = self.x+(self.w/2.2)
                    y = self.y + random.choice([0,-20, -40,20,40])
                    game.SpawnShot(StandardBullet(x,y, 1, 0 ,self.team,id(self),10,1000.0))

            if self.fire_right == True:
                if self.dr <= 0:
                    self.dr = self.gundelay
                    x = self.x-(self.w/2.2)
                    y = self.y + random.choice([0,-20, -40,20,40])
                    game.SpawnShot(StandardBullet(x,y, -1, 0 ,self.team,id(self),10,1000.0))

            if self.fire_up == True:
                if self.du <= 0:
                    self.du = self.gundelay
                    y = self.y+(self.h/2.2)
                    x = self.x + random.choice([0,-20, -40,20,40])
                    game.SpawnShot(StandardBullet(x,y, 0, 1,self.team,id(self),10,1000.0))

            if self.fire_down == True:
                if self.dd <= 0:
                    self.dd = self.gundelay
                    y = self.y-(self.h/2.2)
                    x = self.x + random.choice([0,-20, -40,20,40])
                    game.SpawnShot(StandardBullet(x,y, 0, -1,self.team,id(self),10,1000.0))

                    
                
    def draw(self,engine,data):
        if not self.falling:
            engine.DrawImage(data.images["platform_main"], self.x,self.y,self.w,self.h,1,self.dir.angle,z = -0.1)
            engine.DrawImage(data.images["platform_prop"][int(self.frame)], self.x,self.y,self.w,self.h,1,self.dir.angle,z = -0.2)
        else:
            engine.DrawImage(data.images["platform_dead"], self.x,self.y,self.w*self.fall,self.h*self.fall,1,self.dir.angle, z = (1.0-self.fall)-10)


    def get_hit(self, obj, game):
        if not self.falling:
            game.snd.play_sound("hit " + str(random.randrange(1,7)), "hit")
            self.health -= obj.damage
            if self.health <=0:
                self.falling = True
                game.SpawnPart(particlebase.Flash(0.3))
                game.snd.play_sound("big explosion")
                
    def check_bullets(self, bullets,game):
        if not self.falling:
            for b in bullets:
                if b.team != self.team:
                    if Distance(b.x,b.y,self.x,self.y) < self.cr/3:
                        self.get_hit(b,game)
                        b.on_hit(self,game)

                        d = Vector([self.x - b.x,self.y - b.y])
                        d = d.normalised()
                        game.SpawnPart(particlebase.BulletHit(b.x-d.x,b.y-d.x))
                        
                        if b.life <= 0:
                            bullets.remove(b)




