from worm2d import Engine, Viewport
import pygame, math,random
from utility import*
from shotbase import StandardBullet
from vector import Vector
import particlebase, shipbase


class StandardBadGuy(shipbase.FlyingObject):
    def __init__(self, x, y, dx,dy,w,h,team, home = []):
        super(StandardBadGuy, self).__init__(x, y, w, h, team)
        
        self.speed = 300.0
        self.formspeed = self.speed*1.5

        self.avoidmultiplier = 1

        self.turn_speed = 100

        self.target_range = 1000.0

        self.avoid_range = 100.0

        self.return_range = 150.0

        self.shoot_range = 45.0

        self.target = -1

        self.cr = float(w)/2

        self.angle_to = 0

        self.state = "ROAM"

        self.rand_angle=0

        self.wait = 0

        self.dir = Vector([dx,dy])

        self.health = 20

        self.enemyteams = []

        self.flags = []

        self.image = "fighter"

        self.c = [0,0,1,1]

        for i in range(4):
            if i != self.team:
                self.enemyteams.append(i)


        self.home = home

        if self.home == []:
            self.home = [x,y,500,60]

        self.roamtarget = [x,y]

    def Shoot(self, game):
        if self.wait <= 0:
            sv = Vector([0,0])
            if self.isleader or self.inFormation == False:
                self.isShooting = True
                sv = self.dir
            else:
                sv = Vector([1,0])
                sv = sv.rotated(self.angle_to)
                
            self.wait = 0.2 + float(random.randrange(1,3))/10
            game.SpawnShot(StandardBullet(self.x,self.y, sv.x,sv.y,self.team,id(self),10,1000.0))
            game.snd.play_sound("normal shoot")
                

    def MinionUpdate(self,game,delta):
        if self.mode in ["delta","horiz"]:
            dist = Vector([self.point[0]-self.x, self.point[1]-self.y])
            self.dir = dist.safe_normalised()

            if dist.length >= self.formspeed*delta:
                self.x += self.dir.x*self.formspeed*delta
                self.y += self.dir.y*self.formspeed*delta
            else:
                self.x = self.point[0]
                self.y = self.point[1]
        elif self.mode == "follow":

            dist = Vector([self.point[0]-self.x, self.point[1]-self.y])
            self.dir = dist.normalised()

            #if dist.length >= self.cr:
            self.x += self.dir.x*self.speed*delta
            self.y += self.dir.y*self.speed*delta
                    

        if self.isShooting:
            self.Shoot(game)
            

    def LeaderUpdate(self,game,delta):
        self.wait -= delta
        
        target = None
        
        if self.target == -1:
            t = FindNearest(self, game.ships, self.target_range, self.enemyteams)
            if t[0] != -1:
                self.target = t[0]
                self.state = "ATTACK"
                
        for i in game.ships:
            if self.target == id(i):
                target = i

        if target == None:
            self.target = -1
            self.state = "ROAM"

        if self.state == "ROAM":
            #no targets.
            self.target = -1
            if Distance(self.x,self.y,self.roamtarget[0],self.roamtarget[1]) < self.home[3]:
                r = random.randrange(10, self.home[2])
                v = Vector([r,0]).rotated(random.randrange(0,360))

                
                self.roamtarget[0],self.roamtarget[1] = v.x+self.home[0],v.y+self.home[1]

            target_angle = Vector([self.roamtarget[0]-self.x,self.roamtarget[1]-self.y]).safe_normalised()
            width = self.dir.signed_angle_to(target_angle)
            dr = 0
            if width > 0:
                dr = 1
            elif width < 0:
                dr = -1

            l1 = self.dir.rotated((-self.shoot_range/2.5) + 90)
            l2 = self.dir.rotated((self.shoot_range/2.5) + 90)

            if (l1.dot(target_angle) > 0) and (l2.dot(target_angle) < 0):
                self.dir = target_angle
            else:
                self.dir = self.dir.rotated(self.turn_speed*delta*dr)

            

        if target != None:
            if self.state == "ATTACK":
                #target_angle = Vector([self.x-target.x,self.y-target.y]).normalised()
                target_angle = Vector([target.x-self.x,target.y-self.y]).normalised()
                width = self.dir.signed_angle_to(target_angle)
                dr = 0
                if width > 0:
                    dr = 1
                elif width < 0:
                    dr = -1

                l1 = self.dir.rotated((-self.shoot_range/2.5) + 90)
                l2 = self.dir.rotated((self.shoot_range/2.5) + 90)

                if (l1.dot(target_angle) > 0) and (l2.dot(target_angle) < 0):
                    self.dir = target_angle
                else:
                    self.dir = self.dir.rotated(self.turn_speed*delta*dr)
                
                #f = FindNearest(self, game.ships, self.target_range, self.enemyteams)
                f = [target, Distance(target.x,target.y, self.x,self.y), target.x,target.y,target.dir]
                if f != None:
                    flags = Side(f[4], Vector([f[2],f[3]]), Vector([self.x,self.y]))

                    self.flags = flags

                    if "f" in flags and f[1] < self.avoid_range:
                        self.state = "AVOID"
                        if "r" in flags:
                            self.rand_angle = f[4].rotated(-90)
                        elif "l" in flags:
                            self.rand_angle = f[4].rotated(90)

                    if "b" in flags and f[1] < (self.avoid_range/3):
                        self.state = "AVOID"
                        if "r" in flags:
                            self.rand_angle = f[4].rotated(90)
                        elif "l" in flags:
                            self.rand_angle = f[4].rotated(-90)
                    
            elif self.state == "AVOID":
                if Distance(self.x,self.y, target.x,target.y) > self.return_range:
                    self.state = "ATTACK"
                    self.target = -1
                else:
                    target_angle = self.rand_angle
                    try:
                        width = self.dir.signed_angle_to(target_angle)
                    except:
                        width = 0
                    dr = 0
                    if width > 0:
                        dr = 1
                    elif width < 0:
                        dr = -1

                    self.dir = self.dir.rotated(self.turn_speed*self.avoidmultiplier*delta*dr)


            target_angle = Vector([target.x-self.x,target.y-self.y]).normalised()
            l1 = self.dir.rotated((-self.shoot_range/2) + 90)
            l2 = self.dir.rotated((self.shoot_range/2) + 90)

            if (l1.dot(target_angle) > 0) and (l2.dot(target_angle) < 0):
                self.Shoot(game)
                
            
        self.x += self.dir.x*self.speed*delta
        self.y += self.dir.y*self.speed*delta

        
    def update(self, others, game, delta):
        self.wait -= delta
        if not self.isleader and self.inFormation == True:
            self.MinionUpdate(game,delta)
        if self.inFormation == False or self.isleader:
            self.LeaderUpdate(game, delta)

        self.c = game.cam
        

    def draw(self,engine,data):
        if Distance(self.x,self.y, self.c[0],self.c[1]) < self.c[3]*100:
            engine.DrawImage(data.images[self.image], self.x,self.y,self.w,self.h,1,self.dir.angle-90)




#####################
###Fighter############
#####################

class Fighter(StandardBadGuy):
    def __init__(self, x, y, dx,dy,team):
        super(Fighter, self).__init__(x, y, dx, dy,35,35, team)

        self.formspeed = self.speed*1.5

        self.speed = 100.0

        self.turn_speed = 60

        self.target_range = 800.0

        self.avoid_range = 100.0

        self.return_range = 100.0

        self.shoot_range = 10.0

        self.health = 100

        self.image = "fighter"

        self.cr = float(self.w)/2

    def Shoot(self, game):
        if self.wait <= 0:
            sv = Vector([0,0])
            if self.isleader or self.inFormation == False:
                self.isShooting = True
                sv = self.dir
            else:
                sv = Vector([1,0])
                sv = sv.rotated(self.angle_to)
                
            self.wait = 0.5 + float(random.randrange(1,3))/10
            game.SpawnShot(StandardBullet(self.x,self.y, sv.x,sv.y,self.team,id(self),10,1000.0))
            game.snd.play_sound("normal shoot","shoot")

            

#####################
###maggot############
#####################

class Maggot(StandardBadGuy):
    def __init__(self, x, y, dx,dy,team):
        super(Maggot, self).__init__(x, y, dx, dy,25,25, team)

        self.speed = 300.0
        self.formspeed = self.speed*1.5

        self.turn_speed = 100

        self.target_range = 800

        self.avoid_range = 100.0

        self.return_range = 200.0

        self.shoot_range = 45.0

        self.target = -1

        self.state = "ROAM"

        self.health = 20

        self.image = "maggot"

        self.avoidmultiplier = 3

        self.cr = float(self.w)/2

    def Shoot(self, game):
        if self.wait <= 0:
            sv = Vector([0,0])
            if self.isleader or self.inFormation == False:
                self.isShooting = True
                sv = self.dir
            else:
                sv = Vector([1,0])
                sv = sv.rotated(self.angle_to)
                
            self.wait = 0.8 + float(random.randrange(1,3))/10
            game.SpawnShot(StandardBullet(self.x,self.y, sv.x,sv.y,self.team,id(self),8,1000.0))
            game.snd.play_sound("normal shoot","shoot")




#####################################
#######Heavy Fighter#################
#####################################

class HeavyFighter(StandardBadGuy):
    def __init__(self, x, y, dx,dy,team):
        super(HeavyFighter, self).__init__(x, y, dx, dy, 40,40, team)

        self.speed = 250.0
        self.formspeed = self.speed*1.5

        self.turn_speed = 80

        self.avoid_range = 600.0

        self.return_range = 200.0

        self.shoot_range = 10.0

        self.health = 200

        self.image = "heavy fighter"

        self.cr = float(self.w)/2
        
    def Shoot(self, game):
        if self.wait <= 0:
            sv = Vector([0,0])
            if self.isleader or self.inFormation == False:
                self.isShooting = True
                sv = self.dir
            else:
                sv = Vector([1,0])
                sv = sv.rotated(self.angle_to)
                
            self.wait = 0.2 + float(random.randrange(1,3))/10
            game.SpawnShot(StandardBullet(self.x,self.y, sv.x,sv.y,self.team,id(self),8,1000.0, 20,20))
            game.snd.play_sound("heavy shot")



#####################################
#######Imobilizare###################
#####################################

class Imobilizare(StandardBadGuy):
    def __init__(self, x, y, dx,dy,team):
        super(Imobilizare, self).__init__(x, y, dx, dy, 70,70, team)

        self.speed = 70.0
        self.formspeed = self.speed*1.5

        self.turn_speed = 150

        self.target_range = 500.0

        self.avoid_range = 0.0

        self.return_range = 0.0

        self.shoot_range = 5.0

        self.health = 200

        self.image = "imobilizare"

        self.cr = float(self.w)/2
        
    def Shoot(self, game):
        if self.wait <= 0:
            sv = Vector([0,0])
            if self.isleader or self.inFormation == False:
                self.isShooting = True
                sv = self.dir
            else:
                sv = Vector([1,0])
                sv = sv.rotated(self.angle_to)
                
            self.wait = 2.0 + float(random.randrange(1,3))/10
            game.SpawnShot(StandardBullet(self.x,self.y, sv.x,sv.y,self.team,id(self),10,1000.0, 50,50))
            game.snd.play_sound("heavy shot")

class FuelTanker(StandardBadGuy):
    def __init__(self, x, y, dx,dy,team):
        super(FuelTanker, self).__init__(x, y, dx, dy, 250,250, team)
            
        self.speed = 0.0
        
        self.formspeed = 0

        self.turn_speed = 0

        self.target_range = 0.0

        self.avoid_range = 0.0

        self.return_range = 0.0

        self.shoot_range = 5.0

        self.health = 150

        self.image = "fuel tanker"

        self.cr = float(self.w)/2

    def Shoot(self, game):
        pass
    def get_hit(self, obj, game):
        game.snd.play_sound("hit " + str(random.randrange(1,7)), "hit")
        self.health -= obj.damage
        if self.health <=0:
            self.remove = True
            game.SpawnPart(particlebase.Flash(0.3))
            game.SpawnPart(particlebase.Explosion(self.x,self.y,self.cr*4.5,1.0))
            game.snd.play_sound("big explosion", "major explosion")
    def check_bullets(self, bullets,game):
        for b in bullets:
            if b.team != self.team:
                if Distance(b.x,b.y,self.x,self.y) < self.cr:
                    self.get_hit(b,game)
                    b.on_hit(self,game)

                    d = Vector([self.x - b.x,self.y - b.y])
                    d = d.normalised()
                    game.SpawnPart(particlebase.BulletHit(b.x-d.x,b.y-d.x))
                    
                    if b.life <= 0:
                        bullets.remove(b)

    def draw(self,engine,data):
        engine.DrawImage(data.images[self.image], self.x,self.y,self.w,self.h,1,self.dir.angle-90, z= -1)

class Transport(StandardBadGuy):
    def __init__(self, x, y, dx,dy,team):
        super(Transport, self).__init__(x, y, dx, dy, 200,200, team)
        
        self.speed = 0.0
        
        self.formspeed = 0

        self.turn_speed = 0

        self.target_range = 0.0

        self.avoid_range = 0.0

        self.return_range = 0.0

        self.shoot_range = 5.0

        self.health = 500

        self.image = "transport"

        self.cr = float(self.w)/2
    def Shoot(self, game):
        pass
    def get_hit(self, obj, game):
        game.snd.play_sound("hit " + str(random.randrange(1,7)), "hit")
        self.health -= obj.damage
        if self.health <=0:
            self.remove = True
            game.SpawnPart(particlebase.Explosion(self.x,self.y,self.cr*4.5,1.0))
            game.snd.play_sound("explosion", "major explosion")
    def check_bullets(self, bullets,game):
        for b in bullets:
            if b.team != self.team:
                if Distance(b.x,b.y,self.x,self.y) < self.cr:
                    self.get_hit(b,game)
                    b.on_hit(self,game)

                    d = Vector([self.x - b.x,self.y - b.y])
                    d = d.normalised()
                    game.SpawnPart(particlebase.BulletHit(b.x-d.x,b.y-d.x))
                    
                    if b.life <= 0:
                        bullets.remove(b)

    def draw(self,engine,data):
        engine.DrawImage(data.images[self.image], self.x,self.y,self.w,self.h,1,self.dir.angle-90, z= -1)


        
class SciencePlat(StandardBadGuy):
    def __init__(self, x, y, dx,dy,team):
        super(SciencePlat, self).__init__(x, y, dx, dy, 500,500, 0)
        
        self.speed = 0.0
        
        self.formspeed = 0

        self.turn_speed = 0

        self.target_range = 0.0

        self.avoid_range = 0.0

        self.return_range = 0.0

        self.shoot_range = 5.0

        self.health = 1000

        self.image = "science facility"

        self.cr = float(self.w)/2
    def Shoot(self, game):
        pass
    def get_hit(self, obj, game):
        game.snd.play_sound("hit " + str(random.randrange(1,7)), "hit")
        self.health -= obj.damage
        if self.health <=0:
            self.remove = True
            game.SpawnPart(particlebase.Flash(0.3))
            game.SpawnPart(particlebase.Explosion(self.x,self.y,self.cr*4.5,1.0))
            game.snd.play_sound("big explosion", "major explosion")
    def LeaderUpdate(self,game,delta):
        pass
    def check_bullets(self, bullets,game):
        for b in bullets:
            if b.team != self.team:
                if Distance(b.x,b.y,self.x,self.y) < self.cr/4:
                    self.get_hit(b,game)
                    b.on_hit(self,game)

                    d = Vector([self.x - b.x,self.y - b.y])
                    d = d.normalised()
                    game.SpawnPart(particlebase.BulletHit(b.x-d.x,b.y-d.x))
                    
                    if b.life <= 0:
                        bullets.remove(b)

    def draw(self,engine,data):
        engine.DrawImage(data.images[self.image], self.x,self.y,self.w,self.h,1,self.dir.angle-90, z= -1)




#####################
###Fighter############
#####################

class GoodGuyFighter(StandardBadGuy):
    def __init__(self, x, y, dx,dy,team):
        super(GoodGuyFighter, self).__init__(x, y, dx, dy,35,35, 0)

        self.formspeed = self.speed*1.5

        self.speed = 200.0

        self.turn_speed = 100

        self.target_range = 1500.0

        self.avoid_range = 100.0

        self.return_range = 100.0

        self.shoot_range = 10.0

        self.health = 150

        self.image = "goodguyship"

        self.cr = float(self.w)/2

    def Shoot(self, game):
        if self.wait <= 0:
            sv = Vector([0,0])
            if self.isleader or self.inFormation == False:
                self.isShooting = True
                sv = self.dir
            else:
                sv = Vector([1,0])
                sv = sv.rotated(self.angle_to)
                
            self.wait = 0.5 + float(random.randrange(1,3))/10
            game.SpawnShot(StandardBullet(self.x,self.y, sv.x,sv.y,self.team,id(self),10,1000.0))
            game.snd.play_sound("normal shoot","shoot")
        
