from worm2d import Engine, Viewport
import pygame, math
from utility import*
import sound, shipbase, hud, enemy_ship, data,formation,background, enemy_platform, enemy_cruiser
from vector import Vector


class Game(object):
    def __init__(self, filename, engine, data):

        pygame.event.set_grab(True)
        pygame.mouse.set_visible(False)
        pygame.mouse.set_pos([0, 0])

        
        self.engine = engine
        self.view = Viewport(0,0,1,1, 100.0, 75.0)

        self.data = data

        self.snd = sound.SoundManager(self.data)

        
        self.ships = []
        self.shots = []
        self.debris = []
        self.particles = []
        self.groups = []
        self.formations = []

        self.PlacePlayers(0,0)

        f = open(filename, "r")

        l = f.readlines()

        self.curtain = 0.0

        self.end = -1

        for i in l:
            j = i.strip("\n").split(" ")

            if j[0] == "SHIP":
                name = j[1].replace("+"," ")
                x,y, = float(j[2]),float(j[3])
                dx,dy = -float(j[4]),-float(j[5])

                t = None

                if name == "fighter":
                    t = enemy_ship.Fighter
                if name == "goodguyship":
                    t = enemy_ship.GoodGuyFighter
                if name == "maggot":
                    t = enemy_ship.Maggot
                if name == "heavy fighter":
                    t = enemy_ship.HeavyFighter
                if name == "imobilizare":
                    t = enemy_ship.Imobilizare
                if name == "platform_main":
                    t = enemy_platform.Platform
                if name == "cruiser":
                    t = enemy_cruiser.Cruiser
                if name == "transport":
                    t = enemy_ship.Transport
                if name == "fuel tanker":
                    t = enemy_ship.FuelTanker
                if name == "science facility":
                    t = enemy_ship.SciencePlat

                if name != "cruiser" and name != "platform_main":
                    self.ships.append(t(x,y,dx,dy,1))
                else:
                    self.ships.append(t(x,y,1))
            

        f.close()

	self.cam = [0,0,0, 20.0]


	self.bg = background.Background([("bg_oc_0",0.1,1.0),("bg_oc_1",0.4,2.0), ("bg_oc_1",0.9,3.0)])

	self.ticks = engine.GetTicks()

	self.snd.play_sound("ship_engine", loop = -1)

    def PlacePlayers(self,x,y):
        
        self.ships.append(shipbase.Player(x,y,1,0))
        self.ships.append(shipbase.Player(x,y,1,0))
        self.ships.append(shipbase.Player(x,y,1,0))

        self.ships.append(shipbase.Player(x,y,1,0))
        self.ships.append(shipbase.Player(x,y,1,0))
        self.ships.append(shipbase.Player(x,y,1,0))

        self.ships.append(shipbase.Player(x,y,1,0))
        self.ships.append(shipbase.Player(x,y,1,0))
        self.ships.append(shipbase.Player(y,y,0,-1))


        f = formation.Formation([id(self.ships[8]),
                                id(self.ships[7]),
                                id(self.ships[6]),
                                id(self.ships[5]),
                                id(self.ships[4]),
                                id(self.ships[3]),
                                id(self.ships[2]),
                                id(self.ships[1]),
                                id(self.ships[0])], self.ships,"cont")
        self.formations.append(f)
        self.hud = hud.HUD([id(self.ships[8]),
                                id(self.ships[7]),
                                id(self.ships[6]),
                                id(self.ships[5]),
                                id(self.ships[4]),
                                id(self.ships[3]),
                                id(self.ships[2]),
                                id(self.ships[1]),
                                id(self.ships[0])])
        
    def SpawnShip(self,ship):
        self.ships.append(ship)
    def SpawnShot(self,shot):
        self.shots.append(shot)
    def SpawnDebris(self,debris):
        self.debris.append(debris)
    def SpawnPart(self,part):
        self.particles.append(part)
    
    def ParseEvents(self):
        for event in pygame.event.get():
            for s in self.ships:
                s.get_event(event)
            for f in self.formations:
                f.get_event(event)
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return -1
    def update(self, delta):

        for f in self.formations:
            if f.update(self.ships,self,delta) == -1:
                if id(f) == id(self.formations[0]):
                    #lose
                    self.end = 0
                self.formations.remove(f)

        #update projectiles
        for s in self.shots:
            s.update(delta)
            if s.life <= 0:
                self.shots.remove(s)
        
        
        #update ships
        cnte = 0
        for s in self.ships:
            s.update(self.ships, self, delta)

            s.check_bullets(self.shots,self)

            if s.team == 1:
                    cnte += 1
            
            if s.remove:
                for f in self.formations:
                    if id(s) in f.ids:
                        f.on_ship_die(id(s),self.ships)
                        self.hud.on_ship_die(id(s),self.ships)
                self.ships.remove(s)
                if type(s) == enemy_ship.SciencePlat:
                    #lose
                    self.end = 0
        if cnte == 0:
            #win
            self.end = 1
                    

        for p in self.particles:
            p.update(delta)
            if p.life <= 0:
                self.particles.remove(p)

        self.hud.update(self)

        if self.end != -1:
            self.curtain += 0.5*delta

        if self.curtain >= 1:
            return self.end
        
    def draw(self):
        self.engine.PreRender(0.2, 0.2, 0.2)
        self.engine.SetView(self.view, self.cam[0],self.cam[1],self.cam[3], angle = self.cam[2])

        self.bg.draw(self.cam, self.engine, self.data)        
		
	for s in self.ships:
            s.draw(self.engine,self.data)

        for s in self.shots:
            s.draw(self.engine,self.data)

        for p in self.particles:
            p.draw(self.engine,self.data)

        self.engine.SetView(self.view, 0,0,1)

        self.hud.draw(self)

        if self.curtain > 0:
            self.engine.DrawQuad(0,0,100,75, self.curtain, 0,0,0, 0)

        self.engine.PostRender()
        
    def run(self):
        while self.engine.IsRunning():
            delta = float(self.engine.GetTicks() - self.ticks)/1000.0
            self.ticks = self.engine.GetTicks()
            
            r = self.ParseEvents()
            if r == -1:
                return -1

            r = self.update(delta)
            if r == -1:
                return 0
            elif r == 0 or r == 1:
                return r

            self.draw()
