from worm2d import Engine, Viewport
import pygame, math,random
from utility import*
from shotbase import StandardBullet
from vector import Vector
import particlebase


class FlyingObject(object):
    def __init__(self, x, y, w, h, team):

        self.team = team
        
        self.x = x
        self.y = y
        self.z = 0

        self.dir = Vector((0,1))

        self.w = w
        self.h = h

        self.cr = w

        self.health = 100

        self.speed = 100

        self.remove = False
        self.isleader = False
        self.mode = ""
        self.isShooting = False
        self.inFormation = False
        self.formspeed = self.speed*2.5
        
    def get_event(self,event):
        pass

    def LeaderUpdate(self,game,delta):
        pass
    
    def update(self, others, game, delta):
        pass
    
    def draw(self,engine,data):
	engine.DrawQuad(self.x,self.y,self.w,self.h,1,0,0,0,self.dir.angle)

    def get_hit(self, obj, game):
        game.snd.play_sound("hit " + str(random.randrange(1,7)), "hit")
        self.health -= obj.damage
        if self.health <=0:
            self.remove = True
            game.SpawnPart(particlebase.Explosion(self.x,self.y,self.cr*4.5,1.0))
            game.snd.play_sound("explosion")

    def check_bullets(self, bullets,game):
        for b in bullets:
            if b.team != self.team:
                if LineCircle(Vector([b.lx,b.ly]), Vector([b.x,b.y]),Vector([self.x,self.y]),self.cr):
                    self.get_hit(b,game)
                    b.on_hit(self,game)

                    d = Vector([self.x - b.lx,self.y - b.ly]).normalised()*(self.cr)
                    game.SpawnPart(particlebase.BulletHit(self.x+d.x,self.y+d.y))
                    
                    if b.life <= 0:
                        bullets.remove(b)
            
class MouseShip(FlyingObject):
    def __init__(self, x, y, w, h, team):
        super(MouseShip, self).__init__(x, y, w, h, team)

    def update(self, others, game, delta):
        x,y = pygame.mouse.get_pos()
        self.x,self.y = game.engine.MouseCoordsToWorldCoords(x,y, 0, 0, 10, game.view)
        self.health = 1000

    
    def draw(self,engine,data):
	engine.DrawQuad(self.x,self.y,self.w,self.h,1,0,0,0,0)



class Player(FlyingObject):
    def __init__(self,x,y,dx,dy):
        super(Player, self).__init__(x, y, 30, 30, 0)
        self.dir = Vector([dx,dy])

        self.speed = 200.0
        self.formspeed = self.speed*1.5
        self.turn_speed = 50.0
        self.wait = 0
        self.mx = 0

        self.cr = 25

        self.health = 400

        self.isleader = False

        self.mode = "delta"

        self.point = (0,0)
        self.angle_to = 0

        self.isShooting = False

    def LeaderUpdate(self,game,delta):
        cx,cy = pygame.mouse.get_rel()

        self.dir = self.dir.rotated((float(cx)/640.0)*self.turn_speed)

        if pygame.mouse.get_pressed()[0]:
            self.isShooting = True
            if self.wait <= 0:
                game.SpawnShot(StandardBullet(self.x,self.y, self.dir.x,self.dir.y,self.team,id(self),10,1000.0))
                self.wait = 0.2 + float(random.randrange(1,3))/10
                game.snd.play_sound("normal shoot")
        else:
            self.isShooting = False
            

        self.x += self.dir.x*self.speed*delta
        self.y += self.dir.y*self.speed*delta

        game.cam = [self.x+(self.dir.x*150),self.y+(self.dir.y*150),270 - (self.dir.angle), 10.0]

    def update(self, others, game, delta):
        self.wait -= delta
        if not self.isleader:
            if self.mode in ["delta","horiz"]:
                dist = Vector([self.point[0]-self.x, self.point[1]-self.y])
                self.dir = dist.safe_normalised()

                if dist.length >= self.formspeed*delta:
                    self.x += self.dir.x*self.formspeed*delta
                    self.y += self.dir.y*self.formspeed*delta
                else:
                    self.x = self.point[0]
                    self.y = self.point[1]
            elif self.mode == "follow":

                dist = Vector([self.point[0]-self.x, self.point[1]-self.y])
                self.dir = dist.normalised()

                #if dist.length >= self.cr:
                self.x += self.dir.x*self.speed*delta
                self.y += self.dir.y*self.speed*delta
                    

            if self.isShooting:
                if self.wait <= 0:
                    sv = Vector([1,0])
                    sv = sv.rotated(self.angle_to)
                        
                    game.SpawnShot(StandardBullet(self.x,self.y, sv.x,sv.y,self.team,id(self),10,1000.0))
                    self.wait = 0.2 + float(random.randrange(1,3))/10
                    game.snd.play_sound("normal shoot", "shoot")
                

    def draw(self,engine,data):
        if self.isleader:
            engine.DrawImage(data.images["goodguyship"], self.x,self.y,self.w,self.h,1,self.dir.angle-90)
        else:
            engine.DrawImage(data.images["goodguyship"], self.x,self.y,self.w,self.h,1,self.angle_to-90)
        
    

        
    
